/* ANSI */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
/* POSIX */
#include <getopt.h>

#include "options.h"
#include "version.h"
#include "error.h"
#include "utils.h"
#include "types.h"

#define MAXOPTIONS (20)

#ifndef ETCDIR
#define ETCDIR "/usr/local/etc"
#endif
#define DEFAULTPATH (ETCDIR ":.")

#define BLANKS " \t\f\r\n"
#define SPACES " \t\f"
#define ENDLINE "\r\n"

int TileXSize = 0 ;
int TileYSize = 0 ;

char MenuName [40] = "" ;

static char MyPixmapPath [1024] = "" ;
char * PixmapPath = MyPixmapPath ;

static char MyTilePath [FILENAME_MAX] = "" ;
char * TilePath = MyTilePath ;

bool NewLeaving = true ;

int Options_Argc = 0 ;
char * Options_Argv [MAXOPTIONS] ;

static void Usage (void)
{
    printf (
"Usage: wmmenu [<options>...][-- <dockoptions>...]\n"
"Normal options:\n"
"-m MENUNAME    set the name of the menu file to load from ~/.wmmenu\n"
"-g WxH         set width and height of tile\n"
"-t XPMFILE     set the pixmap used as button bar background\n"
"-O oldleaving  hide bar when leaving bar but not tile\n"
"-v             print version information\n"
"-h             print this help message\n"
""
    ) ;
}

static void SetGeometry (const char * val)
{
    if (sscanf (val, "%dx%d", &TileXSize, &TileYSize) != 2)
        error ("bad tile geometry \"%s\"", val) ;
    else
    if (64 < TileXSize || TileXSize < 0)
        error ("incorrect tile width %d", TileXSize) ;
    else
    if (64 < TileYSize || TileYSize < 0)
        error ("incorrect tile height %d", TileYSize) ;
}

static void SetTilePath (const char * value)
{
    sprintf (MyTilePath, "%.*s", (int)(sizeof MyTilePath)-1, value) ;
}

static void AddPixmapPath (const char * value)
{
    int curLen ;
    int szLeft ;

    curLen = strlen (MyPixmapPath) ;
    szLeft = (sizeof MyPixmapPath) - curLen ;

    /* don't forget to count space for ':' and ending EOS */
    sprintf (MyPixmapPath+curLen, ":%.*s", szLeft-2, value) ;
}

static void InitDefaults (void)
{
    char dftPixmapPath [FILENAME_MAX] ;
    const char * home ;

    if ((home = getenv ("HOME")) != NULL && home[0] != EOS)
    {
        sprintf (dftPixmapPath, "%.*s/.wmmenu",
            (int)(sizeof dftPixmapPath)-10, home) ;
    }
    else
    {
        strcpy (dftPixmapPath, ".") ;
    }

    strcpy (MyPixmapPath, dftPixmapPath) ;
}

static void ParseDefaults (char * text)
{
    char * p ;
    const char * name ;
    const char * value ;

    assert (text != NULL) ;
    p = text ;
    p += strspn (p, BLANKS) ;

    while (*p != EOS) switch (*p++)
    {
        case '#' :
            p += strcspn (p, ENDLINE) ;
            p += strspn (p, BLANKS) ;
            break ;

        default :
            name = p-1 ;
            p += strcspn (p, SPACES) ;
            *p++ = EOS ;
            p += strspn (p, SPACES) ;
            value = p ;
            p += strcspn (p, ENDLINE) ;
            *p++ = EOS ;
            p += strspn (p, BLANKS) ;

            if (streq (name, "geometry")) SetGeometry (value) ;
            else
            if (streq (name, "xpmpath"))
            {
                AddPixmapPath (value) ;
            }
            else
            if (streq (name, "tile"))
            {
                SetTilePath (value) ;
            }
            else error ("unknown setting \"%s\"", name) ;

            break ;
    }
}

extern void Options_ParseDefaults (void)
{
    char pathList [1000] ;
    char path [FILENAME_MAX] ;
    const char * home ;
    FILE * f ;

    if ((home = getenv ("HOME")) != NULL && *home != EOS)
    {
        sprintf (pathList, "%s/.wmmenu:%s", home, DEFAULTPATH) ;
    }
    else
    {
        strcpy (pathList, DEFAULTPATH) ;
    }

    if (File_FindInPath (path, sizeof path, pathList, "defaults") &&
        (f = fopen (path, "r")) != NULL)
    {
        char * text ;

        text = File_ReadAll (f) ;
        fclose (f) ;

        InitDefaults () ;
        ParseDefaults (text) ;
        free (text) ;
    }
}

extern void Options_Parse (int argc, char ** argv)
{
    int opt ;

    assert (argv != NULL) ;
    assert (argc > 0) ;

    Options_Argc = 0 ;
    Options_Argv[Options_Argc++] = MenuName ;

    while ((opt = getopt (argc, argv, "hg:m:t:vO:")) != EOF) switch (opt)
    {
        case 'g' :
            SetGeometry (optarg) ;
            break ;

        case 'm' :
            sprintf (MenuName, "%.*s", (int)(sizeof MenuName)-1, optarg) ;
            break ;

        case 't' :
            sprintf (MyTilePath, "%.*s", (int)(sizeof MyTilePath)-1, optarg) ;
            break ;

        case 'O' :
            if (streq (optarg, "oldleaving")) NewLeaving = false ;
            else printf ("ignoring unknown feature \"%s\"\n", optarg) ;
            break ;

        case 'v' :
            printf ("wmmenu (c) F.COUTANT v%s\n", VERSION) ;
            exit (EXIT_SUCCESS) ;
            break ;

        case 'h' :
            Usage () ;
            exit (EXIT_SUCCESS) ;
            break ;

        case '?' :
            Usage () ;
            exit (EXIT_FAILURE) ;
            break ;
    }

    if (MenuName[0] == EOS)
        error ("no menu file name specified") ;
    if (MyTilePath[0] == EOS)
        error ("no tile pixmap specified") ;

    while (optind < argc && Options_Argc < MAXOPTIONS)
        Options_Argv[Options_Argc++] = argv[optind++] ;
}

extern void Options_SetMenuName (const char * name)
{
    sprintf (MenuName, "%.*s", (int)(sizeof MenuName)-1, name) ;
}
